import axios, { AxiosError } from '@ohos/axios';
import HashMap from "@ohos.util.HashMap";
import PLVJSONUtils from '../../common/PLVJSONUtils';
import { PLVHttpError } from './PLVHttpError';
import PLVLogger from '../log/PLVLogger';
import { PLVRequestSetting } from './PLVRequestSetting';
import { PLVHttpData } from './PLVHttpData';
import PLVUtils from '../../common/PLVUtils';
import { PLVNetUtils } from '../../common/PLVNetUtils';
const TAG = '[PLVHttpRequest]';
export class PLVHttpRequest {
    constructor(x7) {
        this.baseURL = x7;
    }
    ua(w7) {
        this.userAgent = w7;
        return this;
    }
    create(config) {
        const o7 = axios.create();
        o7.defaults.timeout = 10000;
        o7.defaults.readTimeout = 10000;
        o7.defaults.connectTimeout = 10000;
        o7.defaults.baseURL = this.baseURL;
        o7.defaults.headers.common["w1"] = this.userAgent;
        if (config?.formUrlEncoded) {
            o7.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded';
        }
        let cacheKey = '';
        o7.interceptors.request.use((v7) => {
            if (config?.signKey) {
                const data = v7.params ? v7.params : v7.data;
                const map = PLVJSONUtils.object2Map(data);
                map.delete('log');
                data['sign'] = PLVRequestSetting.createSign(map, config.signKey);
            }
            if (PLVHttpRequest.token) {
                const params = v7.params;
                const data = v7.data;
                if (params && Object.keys(params).includes('appId')) {
                    params['token'] = PLVHttpRequest.token;
                    params["z1"] = PLVHttpRequest.token;
                }
                if (data && Object.keys(data).includes('appId')) {
                    data['token'] = PLVHttpRequest.token;
                    data["z1"] = PLVHttpRequest.token;
                }
            }
            if (config?.useCache) {
                cacheKey = this.generateCacheKey(v7);
            }
            PLVLogger.info(TAG, `--> ${this.method} ${this.baseURL}${config?.url}${PLVNetUtils.formatParams(v7?.params)}`);
            PLVLogger.info(TAG, `--> ${PLVNetUtils.formatParams(v7?.data)}`);
            return v7;
        }, (error) => {
            return Promise.reject(error);
        });
        o7.interceptors.response.use((response) => {
            PLVLogger.info(TAG, `<-- ${response.status} ${this.method} ${this.baseURL}${response.config.url}${PLVNetUtils.formatParams(config?.params)} (${response.performanceTiming?.totalTiming}ms)`);
            PLVLogger.info(TAG, `Header: ${PLVJSONUtils.safeStringify(response.headers)}`);
            PLVLogger.info(TAG, `Result: ${PLVJSONUtils.safeStringify(response.data)}`);
            if (config?.aesDecrypted) {
                const u7 = response.data;
                if (u7) {
                    u7['data'] = PLVJSONUtils.safeParse(PLVRequestSetting.decryptDataSync(u7['data']));
                }
                else {
                    PLVLogger.printError(TAG, 'response error', new Error('jsonData undefined'));
                }
            }
            if (config?.fillData) {
                PLVHttpData.fill(config.fillData, response);
            }
            if (cacheKey) {
                PLVHttpRequest.CACHE_DATA_MAP.set(cacheKey, response);
                setTimeout(() => {
                    PLVHttpRequest.CACHE_DATA_MAP.remove(cacheKey);
                }, PLVHttpRequest.CACHE_TIMEOUT);
            }
            return response;
        }, (error) => {
            return Promise.reject(error);
        });
        return o7;
    }
    request(p) {
        return PLVUtils.safePromise((k7, l7) => {
            p.then((r) => {
                const data = r.data;
                if (data) {
                    for (const key of Object.keys(data)) {
                        if (key == 'code' && data[key] != 200 + '') {
                            l7(new PLVHttpError(new AxiosError(PLVJSONUtils.safeStringify(r.data || ''), data[key], undefined, undefined, r)));
                            return;
                        }
                    }
                }
                k7(r);
            }).catch((err) => {
                l7(new PLVHttpError(err));
            });
        });
    }
    get(config) {
        this.method = 'GET';
        return this.checkCache(config) || this.request(this.create(config).get(config?.url, config));
    }
    post(config) {
        this.method = 'POST';
        return this.checkCache(config) || this.request(this.create(config).post(config?.url, config?.data, config));
    }
    checkCache(config) {
        if (config?.useCache) {
            const cacheKey = this.generateCacheKey(config);
            if (PLVHttpRequest.CACHE_DATA_MAP.hasKey(cacheKey)) {
                PLVLogger.info(TAG, `--> ${this.method} ${this.baseURL}${config?.url} use cache`);
                return Promise.resolve(PLVHttpRequest.CACHE_DATA_MAP.get(cacheKey));
            }
        }
        return undefined;
    }
    generateCacheKey(config) {
        let cacheKey = this.baseURL + '' + config?.url;
        const params = config?.params;
        const data = config?.data;
        const h7 = (record) => {
            if (record) {
                for (const key of Object.keys(record)) {
                    if (key == 'timestamp' || key == 'sign' || key == 'token' || key == 'viewerToken') {
                        continue;
                    }
                    if (key == 'appId' && PLVHttpRequest.token) {
                        cacheKey += PLVHttpRequest.token;
                    }
                    else {
                        cacheKey += record[key];
                    }
                }
            }
        };
        h7(params);
        h7(data);
        return cacheKey;
    }
}
PLVHttpRequest.CACHE_TIMEOUT = 1000 * 60 * 5;
PLVHttpRequest.CACHE_DATA_MAP = new HashMap();
